@@ -17,7 +17,7 @@ fn target_type_or_trait_in_impl_block() {
         r#"
 impl Tra$0
 "#,
-        expect![[r##"
+        expect![[r#"
             kw self
             kw super
             kw crate
@@ -27,11 +27,10 @@ impl Tra$0
             st Tuple
             md module
             st Unit
-            ma makro!(…) #[macro_export] macro_rules! makro
+            ma makro!(…) macro_rules! makro
             un Union
-            ma makro!(…) #[macro_export] macro_rules! makro
             bt u32
-        "##]],
+        "#]],
     )
 }
 
@@ -41,7 +40,7 @@ fn target_type_in_trait_impl_block() {
         r#"
 impl Trait for Str$0
 "#,
-        expect![[r##"
+        expect![[r#"
             kw self
             kw super
             kw crate
@@ -51,11 +50,10 @@ impl Trait for Str$0
             st Tuple
             md module
             st Unit
-            ma makro!(…) #[macro_export] macro_rules! makro
+            ma makro!(…) macro_rules! makro
             un Union
-            ma makro!(…) #[macro_export] macro_rules! makro
             bt u32
-        "##]],
+        "#]],
     )
 }
 
@@ -82,6 +80,74 @@ fn after_target_name_in_impl() {
     check(r"impl Trait for Type $0", expect![[r#""#]]);
 }
 
+#[test]
+fn after_struct_name() {
+    // FIXME: This should emit `kw where` only
+    check(
+        r"struct Struct $0",
+        expect![[r#"
+            kw pub(crate)
+            kw pub(super)
+            kw pub
+            kw unsafe
+            kw fn
+            kw const
+            kw type
+            kw impl
+            kw extern
+            kw use
+            kw trait
+            kw static
+            kw mod
+            kw enum
+            kw struct
+            kw union
+            sn tmod (Test module)
+            sn tfn (Test function)
+            sn macro_rules
+            kw self
+            kw super
+            kw crate
+            md module
+            ma makro!(…)           macro_rules! makro
+        "#]],
+    );
+}
+
+#[test]
+fn after_fn_name() {
+    // FIXME: This should emit `kw where` only
+    check(
+        r"fn func() $0",
+        expect![[r#"
+            kw pub(crate)
+            kw pub(super)
+            kw pub
+            kw unsafe
+            kw fn
+            kw const
+            kw type
+            kw impl
+            kw extern
+            kw use
+            kw trait
+            kw static
+            kw mod
+            kw enum
+            kw struct
+            kw union
+            sn tmod (Test module)
+            sn tfn (Test function)
+            sn macro_rules
+            kw self
+            kw super
+            kw crate
+            md module
+            ma makro!(…)           macro_rules! makro
+        "#]],
+    );
+}
+
 #[test]
 fn before_record_field() {
     check(
@@ -93,6 +159,7 @@ struct Foo {
 "#,
         expect![[r#"
             kw pub(crate)
+            kw pub(super)
             kw pub
         "#]],
     )
